-- P1.3/G3_LT.vhdl

-- G. Kemnitz: Technische Informatik 
-- Band 2: Entwurf digitaler Schaltungen
-- 1.3.3 Simulation von Laufzeittoleranzen
--
-- Abb. 1.42: Simulationsmodell der 3-Gatter-Schaltung durch nebenläufige 
-- Signalzuweisungen mit Halte- und Verzögerungszeiten und zum 
-- Vergleich auch auch noch einmal ohne Haltezeit
---------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;


entity G3_LT is end entity;


architecture a of G3_LT is
 -- Signalvereinbarungen wie in Abb. 1.42
 signal x1, x2, x3, x4, z1, z2, y: std_logic;
 constant th : delay_length:= 0.5 ns;
 constant td1: delay_length:= 1 ns;
 constant td2: delay_length:= 2 ns;
 -- zusätzliche Signalvereinbarungen zur Simulation ohne Haltezeit
 signal z1x, z2x, yx: std_logic;

begin

-- Verhaltensbeschreibungen der drei Gatter mit Halte- und Verzögerungszeiten
G1: z1 <= 'X' after th, x1 and x2 after td1;
G2: z2 <= 'X' after th, x3 and x4 after td1;
G3: y  <= 'X' after th, z1 or  z2 after td2;
--------------------------------------------
-- zusätzliche Beschreibung der drei Gatterschaltung ohne Haltezeiten
G1x: z1x <=  x1 and  x2 after td1;
G2x: z2x <=  x3 and  x4 after td1;
G3x: yx  <= z1x or  z2x after td2;



-- Prozess zur Erzeugung der Eingabesignale
Eingabe: process
 begin
  wait for 1 ns;  x3<='1';		-- 1 ns
  wait for 2 ns;  x1<='1'; x4<='1';	-- 3 ns
  wait for 1 ns;  x2<='1';		-- 4 ns
  wait for 2 ns;  x3<='0';		-- 6 ns
  wait for 5 ns;  x2<='0';		--11 ns
  wait for 3 ns;  x4<='0';		--14 ns
  wait for 3 ns;  x1<='0'; x4<='1';	--17 ns
  wait for 2 ns;  x2<='1'; x3<='1';	--19 ns
  wait for 3 ns;  x1<='1';		--22 ns
  wait for 2 ns;  x3<='0';		--24 ns
  wait for 3 ns;  x2<='0';		--27 ns
  wait for 3 ns;  x1<='0'; x4<='0';	--29 ns
  wait for 2 ns;  x3<='1';		--31 ns
  wait;
 end process;

end architecture;

