-- P1.6/BspMealy.vhdl 

-- G. Kemnitz: Technische Informatik 
-- Band 2: Entwurf digitaler Schaltungen
-- 1.6.2 Vom Zustandsgraph zur VHDL-Beschreibung

-- Beispiel für einen Mealy-Automaten 
---------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity BspMealy is end entity;

architecture Test of BspMealy is
 signal T, I, x, y: std_logic;
 signal s: std_logic_vector(1 downto 0);
begin

 process(I, T)
  variable v: std_logic_vector(2 downto 0);
 begin
  if I='1' then
   s <= "00";
  elsif rising_edge(T) then
   v := s & x;
   case v is
    when "00"&'0' | "10"&'0' => s <= "00";
    when "01"&'0' | "00"&'1' => s <= "01";
    when "10"&'1' | "01"&'1' => s <= "10";
    when others               => s <= "XX";
   end case;
  end if;
 end process;

 process(x, s)
  variable v: std_logic_vector(2 downto 0);
 begin
  v := s & x;
 case v is
   when "00"&'1' | "01"&'0' | "10"&'0' => y <= '0';
   when "00"&'0' | "01"&'1' | "10"&'1' => y <= '1';
   when others => y <= 'X';
  end case;
 end process;


 Eingabeprozess: process
  constant x_vec: std_logic_vector:="00100111010";
  variable ct: natural;
 begin
  -- Abbruch nach Abarbeitung aller Eingabewerte
  if ct>x_vec'high then wait; end if;
  -- Aktivierung des Initialisierungssignals im ersten Takt
  if ct=0 then I<='1'; else I<='0'; end if;
  -- Eingabe bereitstellen
  x <= x_vec(ct) after 200 ps;
  -- Takt weiterschalten
  T<='1'; wait for 1 ns; T<='0'; wait for 1 ns;
  -- Zähler weiterschalten
  ct := ct +1;
 end process;

end architecture;

 

