-- P1.6/QuadEnc.vhdl

-- G. Kemnitz: Technische Informatik 
-- Band 2: Entwurf digitaler Schaltungen
-- 1.6.5 Operationsablaufgraph

-- Quadraturencoder als Entwurfseinheit
---------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

library Tuc;
use Tuc.Numeric_Sim.all;

entity QuadEnc is 
 port(T, I: in std_logic;
      ab: in std_logic_vector(1 downto 0);
      ctout: out tSigned(15 downto 0));
end entity;


architecture a of QuadEnc is
 signal I_del: std_logic;
 signal ab_del: std_logic_vector(3 downto 0);
 signal ct: tSigned(15 downto 0);
begin

-- Abtastprozess ohne Initialisierung
 process(T)
 begin
  if rising_edge(T) then
   I_del <= I;
   ab_del(1 downto 0) <= ab;
   ab_del(3 downto 2) <= ab_del(1 downto 0);
  end if;
 end process;

-- Abtastprozess mit Initialisierung
 process(I_del, T)
 begin
  if I_del='1' then
   ct <= "0000000000000000";
  elsif rising_edge(T) then
   case ab_del is
    when "0010" | "0100" |"1011" | "1101" => ct <= ct+"01";
    when "0001" | "0111" |"1000" | "1110" => ct <= ct-"01";
    when "0000" | "0101" |"1010" | "1111" => null;
    when others => ct <="XXXXXXXXXXXXXXXX";
   end case;
  end if;
 end process;
 ctout <= ct;
end architecture;
