-- G. Kemnitz: Technische Informatik 
-- Band 2: Entwurf digitaler Schaltungen
-- 3.1.4 Funktionen
-- 3.1.8  Nebenläufige Prozeduraufrufe
-- 3.1.9 Zusammenfassung und Uebungsaufgaben
--
-- Package mit Vereinbarungen und Beschreibungen von Unterprogrammen
---------------------------------------------------------------------

-- Vorspann
library ieee;
use ieee.std_logic_1164.all;
-- für die Funktion rand
use ieee.math_real; 

library Tuc;
use Tuc.Eingabe.all;
use Tuc.Ausgabe.all; 
use Tuc.Numeric_Sim.all;

package UP_pack is
--======================================================================== 
 -- Funktionsbeschreibung einer kombinatorischen Schaltung
 function fWTab(x: std_logic_vector(2 downto 0)) return std_logic_vector;

-- Uebergangsfunktion eines Spezialzählers zur Erzeugung aller Variationen 
-- von Testeingaben einschliesslich der mit einem einzelnen ungültigen Bit
 function countx(x: std_logic_vector) return std_logic_vector;

--======================================================================== 
 -- Nebenläufig aufzurufende Prozeduren aus Abschn. 3.1.8
 -- Nachbildung eines NAND-Gatters durch eine Prozedur
 procedure nand_proc(signal a, b: std_logic; signal y: out std_logic; 
    th, td: delay_length);

 -- Nachbildung eines Testgeneratorprozesses durch eine Prozedur
 procedure Testgen(signal y: out std_logic_vector; tP: delay_length);

-- Nachbildung eines Prozesses zur Protokollierung von Signalverläufen
-- durch eine Prozedur
 procedure Ausgabe(signal x: std_logic_vector; signal y: std_logic);
--======================================================================== 
end package;


package body UP_pack is
--======================================================================== 
 -- Funktionsbeschreibung einer kombinatorischen Schaltung
 function fWTab(x: std_logic_vector(2 downto 0)) return std_logic_vector is
  variable y: std_logic_vector(2 downto 0);
 begin
  case x is
   when "000"             => y:="100";
   when "001"|"011"|"110" => y:="110";
   when "010"|"101"       => y:="101";
   when "100"             => y:="001";
   when "111"             => y:="011";
   when others            => y:="XXX";
  end case;
  return y;
 end function;

---------------------------------------------------------------------------
 -- Uebergangsfunktion eines Spezialzählers zur Erzeugung von Testeingaben,
 -- die auch en eizelnes ungültiges Bit enthalten können

 function countx(x: std_logic_vector) return std_logic_vector is
  variable y: std_logic_vector(x'range):=x;
 begin
  for idx in x'low to x'high loop
   --write("y'low=" & str(y'low) & " y'high=" & str(y'high))
   if y(idx) = '0' then y(idx):='1'; exit;
   elsif y(idx)='1' and  not is_X(y(y'high downto idx+1)) then 
    y(idx):='X'; exit;
   else y(idx):='0'; 
   end if;
  end loop;
  return y;
 end function;

--======================================================================== 
 -- Nebenläufig aufzurufende Prozeduren aus Abschn. 3.1.8
 -- Nachbildung eines NAND-Gatters durch eine Prozedur
 procedure nand_proc(signal a, b: std_logic; signal y: out
                     std_logic; th, td: delay_length) is
 begin
  y <= 'X' after th;
  if not is_x(a) and not is_x(b) then
   y <= transport a nand b after td;
  end if;
 end procedure;

 -- Nachbildung eines Testgeneratorprozesses durch eine Prozedur
 procedure Testgen(signal y: out std_logic_vector; tP: delay_length) is
  constant Startwert: std_logic_vector(y'range):=(others=>'0');
  variable z: std_logic_vector(y'range):=Startwert;
 begin
  loop
   y <= z;
   z := countx(z);
   wait for tP;
   exit when z=Startwert;
  end loop;
 end procedure;

-- Nachbildung eines Prozesses zur Protokollierung von Signalverläufen
-- durch eine Prozedur
 procedure Ausgabe(signal x: std_logic_vector; signal y: std_logic) is
 begin
  write(rechts(str(now), 10) & ": x=" & str(x) & " y=" & str(y));
 end procedure;

--======================================================================== 

end package body;
