-- G. Kemnitz: Technische Informatik 
-- Band 2: Entwurf digitaler Schaltungen
-- 3.3.4 Zusammenfassung und Uebungsaufgaben
--
-- Package mit den Aufgaben
---------------------------------------------------------------------

-- Vorspann
library ieee;
use ieee.std_logic_1164.all;

use std.textio.all;

library Tuc;
use Tuc.Eingabe.all;
use Tuc.Ausgabe.all; 
use Tuc.Numeric_Sim.all;


package AfgEA_pack is

-- exportierte Vereinbarungen zu Aufgabe 3.10
 type tZustand is (Idle, Start, Run, Ready);
 procedure get(pstr: inout tPString; w: out tZustand);
 function str(x: tZustand; fmt:character:='b') return string;

-- exportierte Vereinbarungen zu Aufgabe 3.11
 subtype tByte is tUnsigned(7 downto 0);
 function str_hex(x: tByte) return string;

--exportierte Vereinbarungen zu Aufgabe 3.12
 procedure Protokoll(signal x: std_logic_vector; file f: text);

--exportierte Vereinbarungen zu Aufgabe 3.13
 type tByteFeld is array (natural range <>) of std_logic_vector(7 downto 0);
 function Init(Dateiname: string; N: natural) return tByteFeld;
end package;

package body AfgEA_pack is
--===================================================================
-- Unterprogramme zu Aufgabe 3.10
 procedure get(pstr: inout tPString; w: out tZustand) is
   constant ptr: POSITIVE:=pstr.pta;
   variable pos: natural;
  begin
   if pstr.Status=ok then
    get_sym(pstr, pos, "Idle Start Run Ready ");
    if pstr.Status=ok then
     w:=tZustand'val(pos);
     return;
    end if;
    pstr.pta:=ptr;
    pstr.err_pos:=ptr;
    assign(pstr.err_msg, "tZustand expected");
   end if;
  end procedure;

 function str(x: tZustand; fmt:character:='b') return string is  
 begin 
  if fmt='t' then return "tZustand"; 
  else return tZustand'image(x);
  end if;
 end function;

--===================================================================

-- zu Aufgabe 3.11
-- Tabelle der Hexadezimalziffern
 constant HexTab:string:="0123456789ABCDEF";
-- zu entwerfende Funktionsbeschreibung
 function str_hex(x: tByte) return string is
 begin
  if is_x(x) then return "XX";
  else
   return HexTab(int(x(7 downto 4))+1) & HexTab(int(x(3 downto 0))+1);
  end if;
 end function;

--===================================================================

-- Prozedur-Beschreibung zu Aufg. 3.12
 procedure Protokoll(signal x: std_logic_vector; file f: text) is
 begin
   write(f, str(now) & " : " & str(x));
 end procedure;

--===================================================================

-- Funktionsbeschreibung zu Aufgabe 3.13
 function Init(Dateiname: string; N: natural) return tByteFeld is
  file f: text open read_mode  is Dateiname;
  variable pstr: tPString;
  variable Wert: integer;
  variable Zeile: positive;
  variable Feld: tByteFeld(0 to N-1);
 begin
  for idx in Feld'range loop
   read(f, pstr);
   get(pstr, Wert, 0, 255);
   assert pstr.Status=ok 
    report "Lesefehler Datei " & Dateiname & "(" & str(Zeile)  &
     ": " & str(pstr.err_pos) &  " : " & str(pstr.err_msg)
    severity error;
   Feld(idx):=std_logic_vector(to_tUnsigned(Wert, 8));
   Zeile:=Zeile+1;
  end loop;
  return Feld;
 end function;


end package body;

