-- G. Kemnitz: Technische Informatik 
-- Band 2: Entwurf digitaler Schaltungen
-- 3.4.8 Prozeduren als Bearbeitungsmethoden
--
-- objektorientiertes FIFO-Modell
---------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

library Tuc;
use Tuc.Ausgabe.all;

package FIFO_pack is

 constant N_Adr: positive:=3; 
 subtype tAdrIdx is integer range 0 to 2**N_Adr-1;
 subtype tDaten is std_logic_vector(7 downto 0);
 type tSpeicher is array (tAdrIdx) of tDaten;

-- Datentyp für das FIFO-Objekt
 type tFIFO is record
  Mem   : tSpeicher;        
  sz, lz: tAdrIdx;       
  leer, voll: boolean;        
 end record;

-- Str-Funktion für FIFO-Objekte
 function str(fifo: tFIFO) return string;

-- Bearbeitungsmethoden
 procedure init(signal fifo: inout tFIFO);
 procedure write(x: tDaten; signal fifo: inout tFIFO);
 procedure read(signal y: out tDaten;signal fifo: inout tFIFO);

-- Prozedur zur Nachbildung eines Taktschrittes
procedure TaktSchritt(signal T: out std_logic);

end package;

package body FIFO_pack is

-- zikulare Zählfunktion für FIFO-Adressen
 function inc(x: tAdrIdx) return tAdrIdx is
 begin
  return (x+1) mod (tAdrIdx'high+1);
 end function;


-- Str-Funktion für FIFO-Objekte
 function str(fifo: tFIFO) return string is
  variable idx: tAdrIdx:= fifo.lz;
  variable s: tString;
 begin
  assign(s, "["&str(fifo.lz)&':'&str(fifo.sz)&'|');
  if fifo.leer then return str(s) & "(leer)]";
  else append(s, str_hex(fifo.Mem(idx))); idx:=inc(idx);
  end if;
  while idx /= fifo.sz loop
   append(s, "," & str_hex(fifo.Mem(idx)));
   idx:=inc(idx);
  end loop;
  if fifo.voll then
   append(s, "(voll)]");
  else append(s, "]");
  end if;
  return str(s);
 end function;


-- Initialisierungsmethode
 procedure init(signal fifo: inout tFIFO) is
 begin
   fifo.lz  <= 0;
   fifo.sz  <= 0;
   fifo.leer<= true;
   fifo.voll<= false;
 end procedure;

-- Schreibmethode
 procedure write(x: tDaten; signal fifo: inout tFIFO) is
 begin
  if not fifo.voll then
   fifo.mem(fifo.sz) <= x;
   fifo.leer<=false;
   fifo.sz<=inc(fifo.sz);
   if inc(fifo.sz)=fifo.lz then fifo.voll<=true; end if;
  end if;
 end procedure;

-- Lesemethode
 procedure read(signal y: out tDaten; signal fifo: inout tFIFO) is
 begin
   if not fifo.leer then
    y <= fifo.mem(fifo.lz);
    fifo.voll<=false;
    fifo.lz<=inc(fifo.lz);
    if inc(fifo.lz)=fifo.sz then fifo.leer<=true; end if;
   end if;
 end procedure;

-- Prozedur zur Nachbildung eines Taktschrittes
 procedure TaktSchritt(signal T: out std_logic) is
 begin
  wait for 5 ns; T<='0'; wait for 5 ns; T<= '1';
 end procedure;

end package body;
