-- G. Kemnitz: Technische Informatik 
-- Band 2: Entwurf digitaler Schaltungen
-- 3.4.3 Register und Latches als nebenläufige Prozeduren
--
-- Package mit einer Prozedur zur Nachbildung der Funktion eines Latches
---------------------------------------------------------------------


-- Vorspann
library ieee;
use ieee.std_logic_1164.all;

use std.textio.all;

library Tuc;
use Tuc.Eingabe.all;
use Tuc.Ausgabe.all; 
use Tuc.Numeric_Sim.all;


package Latch_pack is
-------------------------------------------------------------------------
-- Prozedur zur Nachbildung der Latch-Funktion

 procedure LatchModell(signal x: std_logic_vector; signal E: std_logic; 
   signal y: inout std_logic_vector; th, td, ts: delay_length);

end package;

package body Latch_pack is

--=========================================================================
-- Prozedur zur Nachbildung der Latch-Funktion

 procedure LatchModell(signal x: std_logic_vector; signal E: std_logic; 
   signal y: inout std_logic_vector; th, td, ts: delay_length) is
  variable tE:delay_length;
  constant XX: std_logic_vector(y'range):=(others=>'X');
 begin
  loop
   if E/='0' and y/=x then
    y <= XX after th;
   end if;
   if E='1' then
    y <= transport x after td;
   end if;

   if rising_edge(E) then
    tE:=now;
   elsif falling_edge(E) and (now-tE<ts or x'last_event<ts) then
    y <= XX;
   end if;
   wait on E, x;
  end loop;
 end procedure;


end package body;

