-- G. Kemnitz: Technische Informatik 
-- Band 2: Entwurf digitaler Schaltungen
-- 3.4.3 Blockspeicher; Teilüberschrift: Objektorientierte Modellierung
--       
-- Package mit einer Schreibprozedur und einer Lesefunktion für
-- Schreib-Lese-Speicher
---------------------------------------------------------------------


library ieee;
use ieee.std_logic_1164.all;

library Tuc;
use Tuc.Numeric_Sim.all;
use Tuc.Ausgabe.all;


package MemOOP_pack is
 subtype tDaten is std_logic_vector(7 downto 0);
 type tMem is array (natural range <>) of tDaten;
 subtype tAdr is tUnsigned(3 downto 0);

 function read(Mem: tMem; adr: tAdr) return tDaten;
 procedure write(signal Mem: inout tMem; adr: tAdr; x: tDaten);

end package;


package body MemOOP_pack is
--  Lesefunktion für Blockspeicher
 function read(Mem: tMem; adr: tAdr) return tDaten is
  constant cXX: tDaten:= (others=>'X');
  variable idx: NATURAL;
 begin
  if is_x(adr) then return cXX;
  else
   idx:= int(adr);
   if idx<Mem'low or idx>Mem'high then return cXX;
   else return Mem(idx);
   end if;
  end if;
 end function;
-------------------------------------------------------------------------------------------
-- Schreib-Prozedur für Blockspeicher
 procedure write(signal Mem: inout tMem; adr: tAdr; x: tDaten) is
  constant cXX: tDaten:= (others=>'X');
  variable idx: natural;
 begin
  if is_x(adr) then 
   for idx in Mem'range loop
    Mem(idx)<=cXX;
   end loop;
  else
   idx:= int(adr);
   if idx>=Mem'low and idx<=Mem'high then 
    Mem(idx) <= x;
   end if;
  end if;
 end procedure;

end package body;

