-- G. Kemnitz: Technische Informatik 
-- Band 2: Entwurf digitaler Schaltungen
-- 3.4.1 Auslagerung kombinatorischer Funktionen in Packages
--
-- Package mit Prioritäts-Encoder-Funktion
---------------------------------------------------------------------

-- Vorspann
library ieee;
use ieee.std_logic_1164.all;

library Tuc;
use Tuc.Ausgabe.all; 
use Tuc.Numeric_Sim.all;

package PEnc_pack is

-- Datentyp für den Funktionswert
 type tPEncOut is record
  w: tUnsigned(3 downto 0);
  g: std_logic;
 end record;

-- Beschreibung der kombinatorischen Funktion
 function PrioEnc(x: std_logic_vector) return tPEncOut;

-- Konsatante für ungültig
 constant XX_PEncOut: tPEncOut:=(w=>"XXXX", g=>'X');

-- Testfunktion auf ungültig
 function is_x(x: tPEncOut) return boolean;

-- Str-Funktion für Textausgaben
 function str(x: tPEncOut) return string;
end package;


package body PEnc_pack is
 function PrioEnc(x: std_logic_vector) return tPEncOut is
  variable y: tPEncOut:=(w=>x"0", g=>'0');
 begin
  assert x'high<16 report "Fehler" severity failure;
  for idx in x'high downto x'low loop
   if x(idx)='1' then
    y.w:=to_tUnsigned(idx, y.w'length);
    y.g:='1';
    return y;
   end if;
  end loop;
  return y;
 end function;

 function is_x(x: tPEncOut) return boolean is
 begin
  return is_x(x.w) or is_x(x.g);
 end function;

 function str(x: tPEncOut) return string is
 begin
  return "(w=" & str(x.w) & ", g=" & str(x.g) & ")";
 end function;

end package body;
