-- G. Kemnitz: Technische Informatik 
-- Band 2: Entwurf digitaler Schaltungen
-- 3.4.3 Blockspeicher; Teilüberschrift: Die Kernfunktion eines Schreib-
-- Lese-Speichers als Prozedur
--
-- Simulationsmodell der Kernfunktion eines RAMs
---------------------------------------------------------------------

-- Vorspann
library ieee;
use ieee.std_logic_1164.all;

use std.textio.all;

library Tuc;
use Tuc.Eingabe.all;
use Tuc.Ausgabe.all; 
use Tuc.Numeric_Sim.all;


package RAM_pack is

-- Prozedur zur Nachbildung der Kernfunktion eines Schreib-Lese-Speichers
 procedure ram(signal W: std_logic; signal a: tUnsigned; 
    signal x:std_logic_vector; signal y: out std_logic_vector;
    thy, tdy, tsa, tna, twr, tsx: delay_length);

end package;


package body RAM_pack is
--=========================================================================
-- Prozedur zur Nachbildung der Kernfunktion eines Schreib-Lese-Speichers

 procedure ram(signal W: std_logic; signal a: tUnsigned; 
    signal x:std_logic_vector; signal y: out std_logic_vector;
    thy, tdy, tsa, tna, twr, tsx: delay_length) is
  type tRAM is array (natural range <>) of std_logic_vector(x'range);
  constant XX: std_logic_vector(x'range):=(others=>'X');
  variable RAM: tRAM(0 to 2**a'length-1);
  variable t: delay_length;
 begin
  -- write("RAM'low(0):" & str(RAM'low(1)) & " RAM'high(0)" & str(RAM'high(1)));
  loop
    --write(rechts(str(now),8) & " Schleifeneintritt a=" & str(a) &
    --    " x=" & str(x) & " W=" & str(w)); 
   if W='0' then        
    -- write(rechts(str(now),8) & " W=0");          
    y <= XX after thy;
    if not IS_X(a) then
     y <= transport RAM(int(a)) after tdy;
    end if;
   elsif rising_edge(w) then
    -- write(rechts(str(now),8) & " rising_edge(W)");    
    t:= now;
    y <= XX after thy;
    wait until falling_edge(w);
     -- write(rechts(str(now),8) & " falling_edge(W)");    
 
    t:=now-t;               
    if t>=twr and x'last_event >tsx and not is_x(a) then
     -- write(rechts(str(now),8) & " schreibe " & str(x) & " auf Adr. " & str(a));
     RAM(int(a)) := x; 
       

    elsif not is_x(a) then
     -- write(rechts(str(now),8) & " schreibe XX auf Adr. " & str(a));    

     RAM(int(a)) := XX; 
    end if;
    wait for tna;
    if a'last_event < tsa+t+tna then
         -- write(rechts(str(now),8) & " Invalidiere Speicher ");    

     RAM := (others=>XX); 
    end if;
   end if;
   wait on W, a;
  end loop;
 end procedure;

end package body;

