-- G. Kemnitz: Technische Informatik 
-- Band 2: Entwurf digitaler Schaltungen
-- 3.4.3 Blockspeicher; Teilüberschrift: Festwertspeicher
--
-- Package mit den Vereinbarungen und Unterprogrammen zur 
-- Initialisierung eines Festwertspeichers aus einer Datei
---------------------------------------------------------------------

-- Vorspann
library ieee;
use ieee.std_logic_1164.all;

use std.textio.all;

library Tuc;
use Tuc.Eingabe.all;
use Tuc.Ausgabe.all; 
use Tuc.Numeric_Sim.all;


package ROM_pack is

 subtype tByte is std_logic_vector(7 downto 0);
 type tByteMem is array (natural range <>) of tByte;

 -- Abräumprozedur für 2-stellige Hex-Zahlen
 procedure get_Byte(pstr: inout tPString; w: out tByte);

 -- Initialisierungsfunktion
 function ROM_init(datei_name: string; m: positive) return tByteMem;
end package;


package body ROM_pack is
--====================================================================
 constant HexZiffern: string:= "0123456789abcdef";
 procedure get_Byte(pstr: inout tPString; w: out tByte) is
  constant ptr: positive:=pstr.pta;
  variable z0, z1: natural;
 begin
  skip(pstr, " ");
  get_char(pstr, z1, HexZiffern);
  if pstr.Status=ok then
   get_char(pstr, z0, HexZiffern);
   if pstr.Status=ok then
    w:= std_logic_vector(to_tUnsigned(16*z1+z0, 8));
    return;
   end if;
  end if;
  pstr.Status:=err;
  pstr.err_pos:=pstr.pta;
  assign(pstr.err_msg, "expected characters: "&HexZiffern);
 end procedure;
--====================================================================
 function ROM_init(datei_name: string; m: positive) return tByteMem is
  file f: text open read_mode  is datei_name; 
  variable Mem: tByteMem(1 to m);
  variable ByteNr, ZeilenNr: positive;
  variable pstr: tPString;
 begin
  read(f, pstr); 
  S1: loop
   S2: while pstr.Status=ok and pstr.pta<pstr.pte loop
    --write(str(pstr));
    get_Byte(pstr, Mem(ByteNr));
    --write("ByteNr=" & str(ByteNr) &" ZeilenNr=" & str(ZeilenNr));
    exit S1 when ByteNr=m;
    ByteNr:=ByteNr+1;
   end loop;
   if pstr.Status=ok then
    assert not endfile(f) report "Dateiende Zeile " & str(ZeilenNr)&
      ",ByteNr " & str(ByteNr) severity failure;
    read(f, pstr);
    ZeilenNr:=ZeilenNr+1;
   else
    report "Zeile "&str(ZeilenNr)&" Spalte "&str(pstr.err_pos)&" Byte   "
      &str(ByteNr) & " " & str(pstr.err_msg) severity failure;
   end if;
  end loop;
  return Mem;
 end function;
--====================================================================
end package body;
