-- G. Kemnitz: Technische Informatik 
-- Band 2: Entwurf digitaler Schaltungen
-- 3.4.3 Register und Latches als nebenläufige Prozeduren
--
-- Package mit einer Prozedur zur Nachbildung der Funktion eines Registers
---------------------------------------------------------------------

-- Vorspann
library ieee;
use ieee.std_logic_1164.all;

use std.textio.all;

library Tuc;
use Tuc.Eingabe.all;
use Tuc.Ausgabe.all; 
use Tuc.Numeric_Sim.all;


package Reg_pack is
-------------------------------------------------------------------------
-- nebenläufig aufzurufende Prozedur zur Nachbildung des Verhaltens
-- eines Registers 
 procedure RegModell(signal T, I, E: std_logic; signal x: std_logic_vector;
   aw: std_logic_vector; signal y: out std_logic_vector; td, th, ts, 
   tn: delay_length:=0 fs);

end package;


package body Reg_pack is
--=========================================================================
-- nebenläufig aufzurufende Prozedur zur Nachbildung des Verhaltens
-- eines Registers 

 procedure RegModell(signal T, I, E: std_logic; signal x: std_logic_vector;
   aw: std_logic_vector; signal y: out std_logic_vector; td, th, ts, 
   tn: delay_length:=0 fs) is

  constant XX: std_logic_vector(y'range):=(others=>'X');
 begin
  loop

   if I='1' then
    y <= aw after td; 
   elsif rising_edge(T) and E='1' then
    if x'last_event>ts and E'last_event>ts then
     y<= XX after th, x after td;
     wait for tn;
     if x'last_event<tn or E'last_event<ts then
      y<= XX;
      report  "Nachhaltebedingung verletzt" severity warning;
     end if;
    else
     y<= XX;
     report  "Vorhaltebedingung verletzt"
     severity warning;
    end if;
   end if;
   wait on T, I;

  end loop;
 end procedure;

end package body;

