-- G. Kemnitz: Technische Informatik 
-- Band 2: Entwurf digitaler Schaltungen
-- 3.4.6 Schrittweise Modellentwicklung -- Beispiel UART
--
-- ablauforientierten taktsynchrone Beschreibung
---------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.all;
use std.textio.all;

library Tuc;
use Tuc.Ausgabe.all;
use Tuc.Eingabe.all;
use Tuc.StopSim_pack.all;
--use Tuc.Numeric_Sim.all;

use work.UART2_pack.all;

entity Test_UART2 is end entity;

architecture a of Test_UART2 is
 signal x, y: tByte;
 signal TSend, TEmpf, I, ISend_del, IEmpf_del, Start, Start_del, Busy, TxD, RxD, RxD_del, Rdy, Err: std_logic;
 constant tbit: delay_length:= 10 ns;
 file fEmpf: text open write_mode is "Empf.txt";
begin

-- Erzeugung der Takte und des Initialisierungssignals
 TaktGen(TSend, tBit);
 TaktGen(TEmpf, tBit/16.05);
 I <= '0' after 2 ns, '1' after 8 ns, '0' after 3.1*tBit; 

-- nebenläufige Prozeduraufrufe zur Nachbildung der Funktion von Sender und Empfänger
 Sender(T=>TSend, I=>ISend_del, Start=>Start_del, x=>x, TxD=>TxD, Busy=>Busy);
 Empfaenger(T=>TEmpf, I=>ISend_del, RxD=>RxD_del, y=>y, Ready=>Rdy, Err=>Err);

-- Modellierung der Verbindung durch eine Transportverzögerung
 RxD <= transport TxD after 5 ns; 

-- Abtastung asynchroner Eingaben
 AbtastungSend: process(TSend)
  begin
   if rising_edge(TSend) then
    Start_del <= Start after 5.1 ns;
    ISend_del <= I after 2.4 ns;
   end if;
  end process;

 AbtastungEmpfang: process(TEmpf)
  begin
   if rising_edge(TEmpf) then
    RxD_del <= RxD after 3.2 ns;
    IEmpf_del <= I after 2.7 ns;
   end if;
  end process;

-- Eingabeprozess
 Eingabe: process
 begin
  Start <= '0' after 2 ns, '1' after 45 ns, '0' after 62 ns;
  x <= XX after 15 ns, x"3f" after 18 ns; wait until falling_edge(Busy);
  Start <= '1' after 28 ns, '0' after 73 ns;
  x <= XX after 7 ns, x"79" after 9 ns; wait until falling_edge(Busy);
  wait for 100 ns;
  StopSim("Eingabeprozess beendet.");
 end process;

--------------------------------------------------------
-- Ausgabe
 AusgabeSender: process(TSend)
  variable TabKopf: boolean:=true;
  variable Nr: natural;
 begin
  if TabKopf then
   write("TaktNr|   Zeit  |I|Start|    x   |Busy|TxD");
   TabKopf:=false;
  end if;
  if rising_edge(TSend) then 
   write(rechts(str(Nr),6) & "|" & rechts(str(now),9) & "|" & str(ISend_del) & "|  " & str(Start_del)
    & "  |" & str(x) & "|  " & str(Busy) &  " | " & str(TxD));
   Nr:=Nr+1;
  end if;
 end process;

-- Ausgabe
 AusgabeEmpfang: process(TEmpf)
  variable TabKopf: boolean:=true;
  variable Nr: natural;
 begin
  if TabKopf then
   write(fEmpf,"TaktNr|   Zeit  |I|RxD|    y   |Rdy|Err");
   TabKopf:=false;
  end if;
  if rising_edge(TEmpf) then 
   write(fEmpf, rechts(str(Nr),6) & "|" & rechts(str(now),9) & "|" & str(ISend_del) & "| " & str(RxD_del)
    & " |" & str(y) & "| " & str(Rdy) &  " | " & str(Err));
   Nr:=Nr+1;
  end if;
 end process;


end architecture;
