-- G. Kemnitz: Technische Informatik 
-- Band 2: Entwurf digitaler Schaltungen
-- 3.4.6 Schrittweise Modellentwicklung -- Beispiel UART
--
-- Vergleichernder Test zwischen der ablauforientierten Beschreibung
-- und der synthesefähigen Beschreibung des UART-Senders
---------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;

library Tuc;
use Tuc.Ausgabe.all;
use Tuc.Eingabe.all;
use Tuc.StopSim_Pack.all;
--use Tuc.Numeric_Sim.all;

use work.UART2_pack.all;

entity Test_UART3 is end entity;

architecture a of Test_UART3 is
 signal x: tByte;
 signal TSend, I, ISend_del, Start, Start_del, Busy1, Busy2, TxD1, TxD2: std_logic;
 signal z: tZustand;
 constant tbit: delay_length:= 10 ns;
begin
-----------------------------------------------------------------------
-- Prozesse zur Bereitstellung der Eingabesignale
--  Bereitstellung des Start- und das Dateneingabesignals
 Eingabe: process
 begin
  Start <= '0' after 2 ns, '1' after 45 ns, '0' after 62 ns;
  x <= XX after 15 ns, x"3f" after 18 ns; wait until falling_edge(Busy1);
  Start <= '1' after 28 ns, '0' after 73 ns;
  x <= XX after 7 ns, x"79" after 9 ns; wait until falling_edge(Busy1);
  wait for 100 ns; StopSim;
  wait;
 end process;

--  Bereitstellung des Takts und das Initialisierungssignals
 TaktGen(T=>TSend, tP=>tBit);
 I <= '0' after 2 ns, '1' after 8 ns, '0' after 3.1*tBit; 

--  Abtasten des Takts und des Initialisierungssignals mit dem Sendertakt
 AbtastungSend: process(TSend)
  begin
   if rising_edge(TSend) then
    Start_del <= Start after 5.1 ns;
    ISend_del <= I after 2.4 ns;
   end if;
  end process;

-----------------------------------------------------------------------
-- Test- und Referenzobjekt
--  Senderfunktion als Prozedur (Referenzobjekt)
 Sender(T=>TSend, I=>ISend_del, Start=>Start_del, x=>x, TxD=>TxD1, Busy=>Busy1);

--  synthesefähige Senderbeschreibung (Testobjekt)
 process(TSend, ISend_del)
 begin
  if ISend_del='1' then
   z <=(z=>zStopp, idx=>"000", P=>'0');
  elsif rising_edge(TSend) then
   z <= fz_Sender(Start_del, x, z);
  end if;
 end process;
 
 TxD2 <= f_TxD(x, z);
 Busy2<=f_Busy(z);

-----------------------------------------------------------------------
-- Ausgabe
 AusgabeSender: process(TSend)
  variable TabKopf: boolean:=true;
  variable Nr: natural;
 begin
  if TabKopf then
   write("TaktNr|I|Start|    x   |Busy1|Busy2|TxD1|TxD2| Zustand");
   TabKopf:=false;
  end if;
  if rising_edge(TSend) then 
   write(rechts(str(Nr),6) & "|" & str(ISend_del) & "|  " & str(Start_del) & "  |" & str(x) & "|   " & str(Busy1)
    & " |   " & str(Busy2) & " |  " & str(TxD1) & " |  " &  str(TxD2) & " |" & str(z));
   Nr:=Nr+1;
  end if;
 end process;

end architecture;
