-- G. Kemnitz: Technische Informatik 
-- Band 2: Entwurf digitaler Schaltungen
-- 3.4.6 Schrittweise Modellentwicklung -- Beispiel UART
--
-- ablauforientierte Beschreibung mit Wartezeiten
---------------------------------------------------------------------

-- Vorspann
library ieee;
use ieee.std_logic_1164.all;

use std.textio.all;

library Tuc;
use Tuc.Eingabe.all;
use Tuc.Ausgabe.all; 
use Tuc.Numeric_Sim.all;


package UART1_pack is

 subtype tByte is std_logic_vector(7 downto 0);
 constant XX: tByte:=(others=>'X');

--Prozedur zur Beschreibung des Senders
 procedure Sender(signal Start: std_logic; signal x: tByte;
  signal TxD, busy: out std_logic; tbit: delay_length); 

-- Prozedur zur Beschreibung des Empfängers
 procedure Empfaenger(signal RxD: std_logic;  signal y: out tByte; 
   signal Ready, Err: out std_logic; tbit: delay_length);
end package;



package body UART1_pack is
--=================================================================
 procedure Sender(signal Start: std_logic; signal x: tByte;
  signal TxD, busy: out std_logic; tbit: delay_length) is
  variable P: std_logic;
 begin
  
  loop
   TxD<='1'; Busy<='0';
   wait for tbit;
   wait until Start='1';
   TxD<='0'; Busy<='1'; P:='0';
   wait for tbit;
   for idx in x'reverse_range loop
    TxD<=x(idx);
    P:=P xor x(idx);
    wait for tbit;
   end loop;
   TxD<=P;
   wait for tbit;
  end loop;
 end procedure;
--=================================================================
 procedure Empfaenger(signal RxD: std_logic;  signal y: out tByte; 
   signal Ready, Err: out std_logic; 
   tbit: delay_length) is
  variable P: std_logic;
 begin
  Ready<='0'; Err<='0'; 
  loop
   wait until RxD='0';
   Ready<='0'; Err<='0'; P:='0';
   wait for 1.5*tbit;
   for idx in y'reverse_range loop
    y(idx)<=RxD;
    P:=P xor RxD;
    wait for tbit;
   end loop;
   Ready<='1';
   if P/=RxD then Err<='1'; end if;
   wait for tbit;
   if RxD/='1' then Err<='1'; end if;
   wait for 0.5*tbit;
  end loop;
 end procedure;

end package body;
