-- G. Kemnitz: Technische Informatik 
-- Band 2: Entwurf digitaler Schaltungen
-- 3.4.7 Entwicklung eines CORDIC-Rechenwerks
--
-- Packages zum ersten Entwurf eines CORDIC-Rechenwerkes
---------------------------------------------------------------------


library ieee;
use ieee.math_real.cos;
use ieee.math_real.atan;

package cordic1_pack is
 subtype tDaten is real range -2.0 to 2.0; 
 type tDatenArray is array (natural range <>) of tDaten;

 function "sra"(a: tDaten; b: natural) return tDaten;
 function fAtanTab(N: positive) return tDatenArray;
 function fSCS(Tab: tDatenArray) return tDaten;
end package;

package body cordic1_pack is
 function "sra"(a: tDaten; b: natural) return tDaten is
 begin
  return a/(2.0**b);
 end function;

 function fAtanTab(N: positive) return tDatenArray is
  variable Tab: tDatenArray(0 to N-1);
 begin
  for idx in Tab'range loop
   Tab(idx):=atan(1.0 sra idx);
  end loop;
  return Tab;
 end function;

 function fSCS(Tab: tDatenArray) return tDaten is
  variable w: tDaten:=1.0;
 begin
  for idx in Tab'range loop
   w:=w*cos(Tab(idx));
  end loop;
  return w;
 end function;
end package body;
