-- G. Kemnitz: Technische Informatik 
-- Band 2: Entwurf digitaler Schaltungen
-- 3.5.1 Pseudo-Zufallstest und Mehrversionsvergleich
--
-- Package mit den Prozeduren für einen Mehrversionstest
-- * Testeingabeerzeugung mit der Prozeduren WRTPG 
--   (weighted random test pattern generator)
-- * Takterzeugung mit der Prozedur TaktGen
-- * Vergleich und Protokollierung mit der Prozedur 
--   Vergleicher
----------------------------------------------------------------

use std.textio.all;

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.MATH_REAL;

library Tuc;
use Tuc.Ausgabe.all;

package MVTest_pack is

-- Typenvereinbarungen für den gewichteten Zufallstest 
 subtype tWeight is real range 0.0 to 1.0; 
 type tWeightArray is array (natural range <>) of tWeight;

-- Prozedur zur Nachbildung des Pseudo-Zufallsgenerators
 procedure RTPG(signal T: std_logic; signal y: out std_logic_vector;
    w: tWeightArray:=(0=>0.5); seed1, seed2: positive:=1);

-- Taktgenerator
 procedure TaktGen(signal T: out std_logic; tP, tsim: delay_length);

-- Prozedur für der Vergleich und die Ausgabeprotokollierung
 procedure Vergleicher(signal T: in std_logic; signal x, y_ist, y_soll:
      std_logic_vector; ts: delay_length);

end package;


package body MVTest_pack is
--=====================================================================
 procedure RTPG(signal T: std_logic; signal y: out std_logic_vector;
    w: tWeightArray:=(0=>0.5); seed1, seed2: positive:=1) is
  variable ww: tWeightArray(y'range):=(others=>0.5);
  variable yy: std_logic_vector(y'range);
  variable s1: positive:= seed1;
  variable s2: positive:= seed2;
  variable p: tWeight;
 begin
  if w'length=y'length then ww:=w; end if;
  loop 
   for idx in y'range loop
    math_real.uniform(s1, s2, p);
    if p<ww(idx) then yy(idx):='1';
    else yy(idx):='0'; end if;
   end loop;
   y <= yy;
   wait until rising_edge(T);
  end loop;
 end procedure;  

--=====================================================================
-- Taktgenerator
 procedure TaktGen(signal T: out std_logic; tP, tsim: delay_length) is
 begin
  T<='0';
  while now<tsim loop
   wait for tP/2; T<='1';
   wait for tP/2; T<='0';
  end loop;
 end procedure; 

--=====================================================================
-- Prozedur für der Vergleich und die Ausgabeprotokollierung
 procedure Vergleicher(signal T: std_logic; signal x, y_ist, y_soll:
      std_logic_vector; ts: delay_length) is
  variable TNr: positive;
 begin
  write("Verletzte Vorhaltebedingungen und Vergleichsfehler:");
  loop
   wait until rising_edge(T);
    if y_ist'last_event<ts then
     write("Schritt: "& str(TNr) & " x=" & str(x) & " y_ist'last_even=" 
         & str(y_ist'last_event));
    end if;
    if not is_x(y_soll) and y_ist /= y_soll then
     write("Schritt: " & str(TNr)&" x=" & str(x) & " y_ist=" & str(y_ist) 
        & " y_soll=" & str(y_soll));
    end if;
    TNr:=TNr+1;
  end loop;
 end procedure;

end package body;
 
