-- G. Kemnitz: Technische Informatik 
-- Band 2: Entwurf digitaler Schaltungen
-- 3.5.2 Vorab berechnete Eingaben und Soll-Werte
--
-- Package mit den Prozeduren für einen Soll-Ist-Test
--  Die Testeingaben und die Sollwerte werden aus einer Datei 
--  gelesen. Die Prozedur, die die Daten einliesst, erzeugt 
--  auch ein Takt und ein Initialisierungssignal, um auch 
--  sequentielle Schaltungen testen zu können.
----------------------------------------------------------------

use std.textio.all;

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.MATH_REAL;

library Tuc;
use Tuc.Ausgabe.all;
use Tuc.Eingabe.all;

package SollIstTest_pack is

 procedure SollIstTestProc(signal T, I: out std_logic; signal x: out std_logic_vector; signal y:  
  std_logic_vector; Eingabedatei: string; tp, ts: delay_length);

end package;

package body SollIstTest_pack is

 procedure SollIstTestProc(signal T, I: out std_logic; signal x: out std_logic_vector; signal y: std_logic_vector; Eingabedatei: string; tp, ts: delay_length) is
  file f_in:  text open read_mode is Eingabedatei;
  variable vx: std_logic_vector(x'range);
  variable y_soll: std_logic_vector(y'range);
  variable TNr: positive;
  variable pstr: tPString;
 begin
  I<='1'; T<='0'; wait for tp; I<='0'; T<='1'; 
  loop
   if endfile(f_in) then write("Alle Tests abgearbeitet"); exit; end if;
   read(f_in, pstr);

   skip(pstr); get(pstr, vx);
   if pstr.Status/=ok then
    write("Lesefehler Zeile: "& Str(TNr)&" Spalte: "&str(pstr.err_pos)&" " & str(pstr.err_msg));
    exit;
   end if;
   x <= vx;

   skip(pstr); get(pstr, y_soll);
   if pstr.Status/=ok then
    write("Lesefehler Zeile: "& Str(TNr)&" Spalte: "&Str(pstr.err_pos)&" " & str(pstr.err_msg));
    exit;
   end if;

 -- Takt weiterschalten
   wait for tp/2; T<='0';
   wait for tp/2; T<='1';

 -- Kontrolle der Vorhaltezeit
    if y'last_event<ts then
     write("Schritt: "& str(TNr) & " x=" & str(vx) & " y'last_even=" 
         & str(y'last_event));
    end if;

 -- Soll-Ist-Vergleich
    if not is_x(y_soll) and y /= y_soll then
     write("Schritt: " & str(TNr)&" x=" & str(vx) & " y=" & str(y) 
        & " Sollwert=" & str(y_soll));
    end if;
    TNr:=TNr+1;

  end loop;
  wait;
 end procedure;

end package body;
