-- G. Kemnitz: Technische Informatik 
-- Band 2: Entwurf digitaler Schaltungen
-- 3.5.3 Spezifikationstest
--
-- Package mit den Prozeduren für den Spezifikationstest für
-- eine Ampelsteuerung
----------------------------------------------------------------

use std.textio.all;

library IEEE;
use IEEE.STD_LOGIC_1164.all;

library Tuc;
use Tuc.Ausgabe.all;
 

package SpecTest_pack is
 subtype tAusgabe is std_logic_vector(4 downto 0);
 constant gn_rt: tAusgabe:= "00110";
 constant ge_rt: tAusgabe:= "01010";
 constant rt_rt: tAusgabe:= "10010";
 constant rt_gn: tAusgabe:= "10001";
 constant Start: tAusgabe:= "00000";

 procedure SpecTest_Ampel(signal y: tAusgabe; signal Taste: std_logic; ta, tF, tmin, 
   tmax, twa_max, twf_max: delay_length);

end package;


package body SpecTest_pack is
 procedure SpecTest_Ampel(signal y: tAusgabe; signal Taste: std_logic; ta, tF, tmin, 
   tmax, twa_max, twf_max: delay_length) is
  variable twa, twf, tz: delay_length;
  variable vgy: tAusgabe; -- Vorängerausgabe
  variable Flag: boolean; 
 begin
  vgy:=y; 
  wait on y;
  loop
   if not Flag and rising_edge(Taste) then Flag:=true; twf:=now; end if;
   if y'event then
    tz:=now-tz;
    case vgy is   
     when gn_rt => twa:=now;
      if tz<ta then write(str(now)&": Dauer gn_rt: "& str(tz)); end if; 
     when ge_rt => 
      if tz<tmin or tz>tmax then write(str(now)&": Dauer rt_gn: "& str(tz)); end if;
     when rt_rt => 
      if tz<tmin or tz>tmax then write(str(now)&": Dauer rt_gn: "& str(tz)); end if; 
     when rt_gn => Flag:=false;
      if tz<0.9*tF or tz>1.1*tF then write(str(now)&": Dauer rt_gn: "& str(tz)); end if; 
     when Start =>
      if tz>tmax then write(str(now)&": Dauer Start: "& str(tz)); end if; 
     when others => null;
    end case;
    case y is  
     when gn_rt => if not( vgy=ge_rt) then 
      write(str(now) & ": Vorgaenger von gn_rt: " & str(vgy)); end if;
      if twa-now>twa_max then write(str(now)&" Autowartezeit auf gn_rt:"&str(twa)); end if; 
     when ge_rt => if not( vgy=gn_rt or vgy=rt_rt or vgy=Start) then 
      write(str(now)&": Vorgaenger von ge_rt: "&str(vgy)); end if;
     when rt_rt => if not( vgy=ge_rt or vgy=rt_gn) then 
      write(str(now)&": Vorgaenger von rt_rt: "&str(vgy)); end if;
     when rt_gn => if not( vgy=rt_rt) then 
      write(str(now)&": Vorgaenger von rt_gn: "&str(vgy)); end if;
      if twf-now>twf_max then write(str(now) & ": Fussgaengerwartezeit auf rt_gn:" 
       & str(twf)); end if; 
     when Start => null;
     when others => write(str(now)&" unzulaessiger Ausgabewert");
    end case;
    vgy:=y;
   end if;
   if not Flag and rising_edge(Taste) then Flag:=true; twf:=now; end if;
   tz:= now;
   wait on y, Taste;
  end loop;
 end procedure;

end package body;
