-- G. Kemnitz: Technische Informatik 
-- Band 2: Entwurf digitaler Schaltungen
-- 4.2.1 Zeitdiskrete Simulation
--
-- Package für die zeitdiskrete Simulation eines Ringinverters
----------------------------------------------------------------


package PhySim_pack is

 type tSpannung is range -10.0 to 10.0;    -- in Volt
 type tStrom is range -0.1 to 0.1;         -- in A
 type tKapazitaet is range 0.0 to 1.0E-10; -- in Farad
 type tBeta is range -0.1 to 0.1;          -- in A pro V^2

 function ID_NMOS(uGS, uDS, UTN: tSpannung; beta_N: tBeta) return tStrom;
 function ID_PMOS(uGS, uDS, UTP: tSpannung; beta_P: tBeta) return tStrom;

end package;

package body PhySim_pack is

 function ID_NMOS(uGS, uDS, UTN: tSpannung; beta_N: tBeta) return tStrom is
  variable i: tStrom := 1.0;
  variable vuGS: tSpannung := uGS;
  variable vuDS: tSpannung := uDS;
 begin
  assert UTN>=0.0 report "UT hat falsches Vorzeichen" severity failure;
  assert beta_N > 0.0 report "beta hat falsches Vorzeichen" severity failure;
  if uDS<0.0 then -- Rollentausch zwischen Drain und Source
 	vuGS:=vuGS-vuDS;
       	vuDS:=-vuDS;
 	i:=-i;
  end if;
  if vuGS<=UTN then
   return 0.0;
  elsif vuGS-vuDS< UTN then
   return tStrom(beta_N)/2.0*tStrom(vuGS-UTN)**2*i;
  else
   return tStrom(beta_N)*tStrom((vuGS-UTN)*vuDS-(vuDS**2)/2.0)*i;
  end if;
 end function;
-------------------------------------------------------------------------
function ID_PMOS(uGS, uDS, UTP: tSpannung; beta_P: tBeta) return tStrom is
begin
 return -ID_NMOS(-uGS, -uDS, -UTP, -beta_P);
end function;
end Package body;

