-- G. Kemnitz: Technische Informatik 
-- Band 2: Entwurf digitaler Schaltungen
-- 4.2.5 Simulation mit geschalteten Transistorbreiten
--
-- Package zur Simulation mit geschalteten Transistorbreiten
----------------------------------------------------------------


library ieee;
use ieee.std_logic_1164.all;

library Tuc;
use Tuc.Ausgabe.all;


package bRel_pack is

 type tRelB is record
   bmax: real; 
   bmin: real; 
 end record;

 function str(x: tRelB) return string;

 function To_RelB(x: std_logic; b: real) return tRelB;
 function "or"(x1, x2: tRelB) return tRelB;
 function "and"(x1, x2: tRelB) return tRelB;
end package;


package body bRel_pack is

 function str(x: tRelB) return string is
 begin
  return "[" & str(x.bmin) & ":" & str(x.bmax) & "]";
 end function;

 function To_RelB(x: std_logic; b: real) return tRelB is
 begin
  case x is
   when '0'   => return (bmax => 0.0, bmin => 0.0);
   when '1'   => return (bmax =>  b, bmin => b);
   when others => return (bmax => b, bmin => 0.0);
  end case;
 end function;

 function "or"(x1, x2: tRelB) return tRelB is
 begin
  return (bmax=>x1.bmax+x2.bmax, bmin=>x1.bmin+x2.bmin);
 end function;

 function KehrwertAdd(a, b: real) return real is
 begin 
  if a=0.0 or b=0.0 then return 0.0; end if;
  return a*b/(a+b);
 end function;

 function "and"(x1, x2: tRelB) return tRelB is
  variable res: tRelB;
 begin
  return (bmax => KehrwertAdd(x1.bmax, x2.bmax), 
          bmin => KehrwertAdd(x1.bmin, x2.bmin));

 end function;

end package body;
