-- G. Kemnitz: Technische Informatik 
-- Band 2: Entwurf digitaler Schaltungen
-- 4.4.3 Assoziativspeicher
--
-- Package zur objektorientierten Beschreibung eines Assoziativspeichers
---------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

library Tuc;
use Tuc.Numeric_Sim.all;
use Tuc.Ausgabe.all;

package AsRAM_pack is
 subtype tDaten is std_logic_vector(7 downto 0);
 type tMem is array (natural range <>) of tDaten;
 subtype tAdr is tUnsigned(3 downto 0);

-- Lesefunktion, Schreib- und Suchmethode
 function read(Mem: tMem; adr: tAdr) return tDaten;
 procedure write(signal Mem: inout tMem; adr: tAdr; x: tDaten);
 procedure search(x: tDaten; signal  adr: out tAdr;
   signal hit: out std_logic; signal Mem: inout tMem);

end package;


package body AsRAM_pack is
--  Lesefunktion für Blockspeicher
 function read(Mem: tMem; adr: tAdr) return tDaten is
  constant cXX: tDaten:= (others=>'X');
  variable idx: NATURAL;
 begin
  if is_x(adr) then return cXX;
  else
   idx:= int(adr);
   if idx<Mem'low or idx>Mem'high then return cXX;
   else return Mem(idx);
   end if;
  end if;
 end function;
-------------------------------------------------------------------------------------------
-- Schreib-Prozedur für Blockspeicher
 procedure write(signal Mem: inout tMem; adr: tAdr; x: tDaten) is
  constant cXX: tDaten:= (others=>'X');
  variable idx: natural;
 begin
  if is_x(adr) then 
   for idx in Mem'range loop
    Mem(idx)<=cXX;
   end loop;
  else
   idx:= int(adr);
   if idx>=Mem'low and idx<=Mem'high then 
    -- write("schreibe " & str(x) & " auf adr. " & str(idx));
    Mem(idx) <= x;
   end if;
  end if;
 end procedure;

-- Suchmethode
 procedure search(x: tDaten; signal  adr: out tAdr;
   signal hit: out std_logic; signal Mem: inout tMem) is
 begin
  for idx in Mem'range loop
   -- write("idx=" & str(idx) & " Mem(idx)=" & str(Mem(idx), 'h') & " x=" & str(x, 'h'));
   if Mem(idx)=x then
    hit<='1';
    adr<=to_tUnsigned(idx, adr'length);
    return;
   end if;
  end loop;
  -- write("nicht enthalten");
  hit<='0';
  adr<=(others=>'0');
 end procedure;

end package body;

