-- G. Kemnitz: Technische Informatik 
-- Band 2: Entwurf digitaler Schaltungen
-- 5.2 FIR-Filter mit Blockspeichern, Untersabschn. 5.2.3 und 5.2.4
-- 
--
-- Testrahmen für das erste Simulationsmodell des FIR-Filters
--
--  * Beschreibung des Ablaufs mit Schleifen und Fallunterscheidungen
--  * Lesen der Eingabewerte aus einer Datei
--  * tabellarische Ausgabe des Schaltungszustands auf die Standardausgabe
--  * eine graphische Visualisierung der Signalverläufe ist gleichfalls
--    möglich 
---------------------------------------------------------------------

use STD.TEXTIO.TEXT;
use STD.TEXTIO.endfile;

library Tuc;
use Tuc.Eingabe.all;
use Tuc.Ausgabe.all;
use work.FIR1_pack.all;

entity Test_FIR1 is begin end entity;

architecture a of Test_FIR1 is
 constant cROM: tMem(tAdr):=(0.2, 0.5, -0.5, -0.2);
 signal xRAM: tMem(tAdr);
 signal pp: tPipeline; 
 file Eingabedatei: TEXT open READ_MODE is "Eingabe.txt";
 constant tp: DELAY_LENGTH:= 10 ns;
begin
 process
  variable x: tDat;
  variable initRdy: boolean; 
  variable s: tString;
 begin

 -- Initialisierung, Ausgabe Tabellenkopf und erste Tabellenzeile
  pp.cAdr<=1; pp.xAdr<=tAdr'high; wait for tp;
  write("Eingabe:" & StatusKopfText & "Ausgabe:");

 -- Beschreibung der Initialisierungsphase 
  while pp.cAdr/=0 loop
   read(Eingabedatei, x); 
   write(str(x) & str(pp, xRAM));
   InitStep(x, xRAM, pp);
   wait for tp;
  end loop;

 -- Endlosschleife für Verarbeitungsschritte; wenn das Ende der 
 -- Eingabedatei erreicht ist, legt sich der prozess schlafen
 -- und die Simulation endet
  loop
   if pp.cAdr=0 then 
    if endfile(Eingabedatei) then wait; end if;
    read(Eingabedatei, x); assign(s, str(x));
   else
    assign(s, "        ");
   end if;
   append(s, str(pp, xRAM));
   if pp.cAdr=2 then
   -- Ausgabe erst ab dem zweiten Schleifendurchlauf
    if not InitRdy then InitRdy:=True;
    else append(s, str(pp.Akku));
    end if;
   end if;
   write(s);
   NormalStep(x, cROM, xRAM, pp);
   wait for tp;
  end loop;
 end process;
end architecture;

