-- G. Kemnitz: Technische Informatik 
-- Band 2: Entwurf digitaler Schaltungen
-- 5.2 FIR-Filter mit Blockspeichern, ab Untersabschn. 5.2.5
-- 
--
-- Testrahmen für den FIR-Filter mit Wertedarstellung durch Bitvektoren 
-- und Ein- und Ausgabe über Signale
--
--  * Beschreibung des FIR-Filters durch kombinatorische Ausgabefunktionen 
--    und einen Abtastprozess mit asynchroner Initialisierung und taktsyn-
--    chroner Uebergangsfunktion
--
--  * Bereitstellung der Eingabesignale (Takt, Init und Daten) mit einer 
--    nebenläufigen Prozedur, die die Daten aus einer Datei einliest.
--
--  * taktsynchrone tabelarische Ausgabeprotokollierung auf der Standardausgabe
--    mit einer nebenläufigen Prozedur
---------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;

library Tuc;
use Tuc.Eingabe.all;
use Tuc.Ausgabe.all;
use Tuc.Numeric_Sim.all;
use work.FIR2_pack.all;

entity Test_FIR2 is begin end entity;

architecture a of Test_FIR2 is
 constant cROM: tMem:=(to_tKoeff(0.2), to_tKoeff(0.5), to_tKoeff(-0.5),to_tKoeff(-0.2));
 signal xRAM: tMem(0 to M-1);
 signal pp: tPipeline; 
 signal T, I, rd, wr: std_logic;
 signal x: tDat;
begin
  
 Testdatenquelle("Eingabe.txt", rd, T, I, x);
 Testausgabe(T, rd, wr, pp, x, xRAM);

 process(T, I)
 begin
 -- Abtastprozess mit asynchroner Initialisierung und Uebergangsfunktion
  if I='1' then 
   pp.cAdr<=to_tUnsigned(1, tAdr'LENGTH); 
   pp.xAdr <= to_tUnsigned(M-1, tAdr'LENGTH);
   pp.Zustand<=Init;
  elsif rising_edge(T) then
   PipeStep(x, cROM, xRAM, pp);
  end if;
 end process;

 -- Ausgabefunktionen
 rd <= RdGnt(pp);
 wr <= WrGnt(pp);
end architecture;

