-- G. Kemnitz: Technische Informatik 
-- Band 2: Entwurf digitaler Schaltungen
-- 5.2 Points-of-Interest-Berechnung
-- Datei: P5.3/POI_pack.vhdl

library ieee;
use ieee.std_logic_1164.all;

library Tuc;
use Tuc.Numeric_Sim.all;

package POI_pack is

 -- Berechnung der Uebereinstimmungsmasse
 constant N_Mask: natural:=12;
 type tOperation is(add, sub, keine_Operation);
 subtype tFilterIdx is natural range 0 to N_Mask-1;
 type tKoeff is array (tFilterIdx) of tOperation;

 constant cDatenbreite: natural:= 8; 
 subtype tDaten is tSigned(cDatenbreite-1 downto 0); 
 subtype tAkku is tSigned(cDatenbreite+3 downto 0);

  function FilterAdd(Akku: tAkku; x: tDaten; s: tOperation)
   return tAkku;

 type tAkkFeld is array (tFilterIdx) of tAkku;

 procedure AllFilterAdd(signal AkkuF: inout tAkkFeld;
      x: tDaten; c: tKoeff);

 -- Sortier-Pipeline für Uebereinstimmungsmasse

 type tFilterDat is record
  FNr: tFilterIdx; 
  betrag: tAkku;   
  s: std_logic;    
 end record;
 constant cFDat_leer: tFilterDat:=(FNr=>0, Betrag=>x"000", s=>'0');

 procedure SortUeMass(AkkuF: tAkkFeld; signal Max1, Max2: inout tFilterDat);

 -- Klassifikation
 constant Spaltenanzahl: natural:=300;
 constant Zeilenanzahl: natural:=200;

 type tKlassPkt is record
  FNr1, FNr2: tFilterIdx;
  s1,s2: std_logic;
  Betrag: tAkku; 
  i: integer range 2 to Spaltenanzahl-2;
  j: integer range 2 to Zeilenanzahl-2;
 end record;

 function Klassifikation(Max1, Max2: tFilterDat) return tKlassPkt;
 
 -- Sortieren der klassifizierten Punkte

 type tPtrKlassFeld is array (natural range <>) of tKlassPkt;

 constant Pkt_leer: tKlassPkt:=(FNr1=>0, s1=>'0', FNr2=>0,
     s2=>'0', Betrag => x"000", i=>0, j=>0);

end package;

package body POI_pack is

 function FilterAdd(Akku: tAkku; x: tDaten; s: tOperation)
   return tAkku is
 begin
  case s is
   when add => return Akku+x;
   when sub => return Akku-x;
   when others => return Akku;
  end case;
 end function;

 procedure AllFilterAdd(signal AkkuF: inout tAkkFeld;
      x: tDaten; c: tKoeff) is
 begin
  for idx in tKoeff'range loop
   AkkuF(idx) <= FilterAdd(AkkuF(idx), x, c(idx));
  end loop;
 end procedure;

 function Vorzeichen(x: tAkku) return std_logic is
 begin
  return x(x'high);
 end function;

 constant tp: delay_length:= 10 ns;

 procedure SortUeMass(AkkuF: tAkkFeld; signal Max1, Max2: inout tFilterDat) is 
 begin
  Max1 <= (FNr=>0, betrag=>abs(AkkuF(0)), s =>Vorzeichen(AkkuF(0))); 
  Max2 <= cFDat_leer; 
  wait for tP; 
  for idx in 1 to AkkuF'high loop
   if abs(AkkuF(idx))>Max1.betrag then
    Max1<= (FNr=>idx, betrag=>abs(AkkuF(Idx)), s=>Vorzeichen(AkkuF(idx)));
    Max2<=Max1;
   elsif abs(AkkuF(idx))>=Max2.Betrag then
    Max2<= (FNr=>idx, betrag=>abs(AkkuF(Idx)), s=>Vorzeichen(AkkuF(idx)));
   end if;
   wait for tp; 
  end loop;
 end procedure;

 function Klassifikation(Max1, Max2: tFilterDat) return tKlassPkt is
 begin
  if (Max1.Betrag sra 1)> Max2.Betrag then
   return (FNr1=>Max1.FNr, s1=>Max1.s, FNr2=>Max1.FNr,
    s2=>Max1.s, Betrag => Max1.Betrag+(Max1.Betrag sra 1), i=>13, j=>11);
  else 
   return (FNr1=>Max1.FNr, s1=>Max1.s, FNr2=>Max2.FNr,
     s2=>Max2.s, Betrag => Max1.Betrag+Max2.Betrag, i=>13, j=>11);
  end if; 
 end function;

end package body;
 


