----------------------------------------------------------------
-- G. Kemnitz: Technische Informatik 
-- Band 2: Entwurf digitaler Schaltungen
-- Bibliothek Tuc
----------------------------------------------------------------
-- Package Tuc.Zufallstest
-- 
-- definiert Funktionen und Prozeduren zur Bereitstellung pseudo-
-- zufaelliger Testeingaben. 
--
-- 15.02.2010, G. Kemnitz
----------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.MATH_REAL;
library Tuc;
use Tuc.Ausgabe.all;

package Zufallstest is

-- globalen Zustandsvariable, Str-Funktion fuer die Anzeige des Zustands
type tRandState is array(0 to 1) of POSITIVE;
shared variable shared_variable_RandStat: tRandState;
impure function str_RandStat return string;

-- 
 subtype tWeight is real range 0.0 to 1.0; 
 type tWeightVec is array (natural range <>) of tWeight;

impure function rand return real;
impure function rand(w: tWeight; px: tWeight:=0.0) return std_logic;
impure function rand(x: std_logic; px: tWeight:=0.0) return std_logic;
impure function rand(wVec: tWeightVec; px: tWeight:=0.0) return std_logic_vector;
impure function rand(xVec: std_logic_vector; px: tWeight:=0.0) return std_logic_vector;
 

end package;

package body Zufallstest is
-- Anzeige des Zustands des Pseudo-Zufallsgenerators
 impure function str_RandStat return string is
 begin
  return "Startwert des Pseudo-Zufallsgenerators: " & str(shared_variable_RandStat(0)) &
    ", " & str(shared_variable_RandStat(1)); 
 end function;

 ---------------------------------------------------------------
 -- Erzeugung einer Zufallszahl im Bereich von 0.0 bis 1.0
 impure function rand return real is
  variable y: REAL;
 begin
  MATH_REAL.UNIFORM(shared_variable_RandStat(0), shared_variable_RandStat(1), y);
  return y;
 end function;

 -- Erzeugen einen Zufallsbits; px ist die Wahrscheinlich, dass der Wert 'X'
 -- ist und w die Wahrscheinlichkeit, dass er sonst '1' ist 
 impure function rand(w: tWeight; px: tWeight:=0.0) return std_logic is
  constant p: real:=rand*(1.0+px);
 begin
  if p<w then return '1';
  elsif p<1.0 then return '0';
  else return 'X';
  end if;
 end function;
 
 -- Erzeugen eines Zufallsbits mit w=0.5
 impure function rand(x: std_logic; px: tWeight:=0.0) return std_logic is
 begin
  return rand(0.5, px);
 end function;

 -- Erzeugen eines Vektors mit Zufallsbits
 impure function rand(wVec: tWeightVec; px: tWeight:=0.0) return std_logic_vector is
  variable y: std_logic_vector(wVec'RANGE);
 begin
  for idx in wVec'RANGE loop
   y(idx) := rand(wVec(idx), px);
  end loop;
  return y;
 end function;

 -- Erzeugen eines Vektors mit Zufallsbits mit w=0
 impure function rand(xVec: std_logic_vector; px: tWeight:=0.0) return std_logic_vector is
  variable y: std_logic_vector(xVec'RANGE);
 begin
  for idx in xVec'RANGE loop
   y(idx) := rand(0.5, px);
  end loop;
  return y;
 end function;

end package body;
 
